#!/usr/bin/env python
# -*- coding: utf8 -*-

"""
   lftp_mirror.py: This script mirrors a remote FTP server dir with a local dir
"""

# ==============================================================================
# This Script mirrors a remote FTP server dir with a local dir (or vice versa)
# and stores a daily compressed copy of local dir
#
# The real job is done by the marvelous lftp program by Alexander V. Lukyanov
# and it's necessary to run this script
# http://lftp.yar.ru/
#
# This script can be executed in three ways:
#
#   a) shell: command line interactive
#   b) cron: as a programmed task
#   c) cfg: importing arguments from a config file
#
# In the first way, you need to give the required arguments (site, remote FTP
# directory and local directory) to the script in the command line. Optional
# arguments, obviously, are optional.
#
# As a programmed task, don't need supply the arguments in the command line.
# It takes the scripts parameters (previously defined) as default command line
# arguments. These defines the required and several optional arguments (which
# can left blank): user, password, port and options (the other). It's useful
# for set up a cron programmed task with a single command line argument, 'cron'
#
# Finally, the last way is ideal for running multiple mirror operations. With a
# one configuration file can set up various mirror actions with different sites
# or directories, and do all of them in a single script execution. Arguments
# are imported from this config file, with a section for each mirror operation,
# and there are not limits for sections.
#
# For further information visits the lftp_mirror's website:
#     http://joedicastro.com
#
# ==============================================================================

# ==============================================================================
#    Copyright 2010 joe di castro <joe@joedicastro.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ==============================================================================


__author__ = "joe di castro - joe@joedicastro.com"
__license__ = "GNU General Public License version 3"
__date__ = "16/9/2012"
__version__ = "0.15"

try:
    import sys
    import os
    import glob
    import base64
    import time
    import datetime
    import re
    import tarfile
    import platform
    import socket
    import smtplib
    import getpass
    import config_utils
    import logging
    from argparse import ArgumentParser, SUPPRESS
    from ConfigParser import SafeConfigParser
    from subprocess import Popen, PIPE, STDOUT
    from email.mime.text import MIMEText
    from email.mime.multipart import MIMEMultipart
    from email.utils import COMMASPACE, formatdate
    from crontab import CronTab
    from ftp_error_codes import ftp_error_codes
except ImportError:
    # Checks the installation of the necessary python modules
    print((os.linesep * 2).join(["An error found importing one module:",
                                 str(sys.exc_info()[1]), "You need to install it", "Exit..."]))
    sys.exit(-2)


def arguments():
    """Defines the command line arguments for the script."""
    main_desc = ("Mirror a remote FTP directory into a local directory or vice"
                 " versa through the lftp program")
    subs_desc = "Select a running mode from the following:"
    epilog = ("For detailed help for each mode, select a mode followed by help"
              " option, e.g.:{0}{0}%(prog)s shell -h").format(os.linesep)
    cron_use = "%(prog)s [-h]"
    shell_use = ("%(prog)s site remote local [options]{0}{0}By default "
                 "downloads the changes from remote FTP directory to local "
                 "directory.{0}To upload changes from local to remote FTP, use"
                 " the 'r, --reverse' option").format(os.linesep)
    file_use = ("%(prog)s config_file [-h]{0}{0}The structure of the "
                "config file (a simple text file) is as follows:{0}{0}[section]"
                "{0}site = {{ftp server URL or IP}}{0}port = (ftp server port)"
                "{0}remote = {{remote directory}}{0}local = {{local directory}}"
                "{0}user = (ftp server username){0}password = (user password "
                "encoded in base64){0}options = (other options){0}{0}Section is"
                " a name that defines the mirror operation. Usually is the ftp "
                "server's name or directory' name. Useful for distinguish one "
                "mirror operation from others. Write one section for each "
                "mirror action with no limits in the number of sections.{0}{0}"
                "Values between curly brackets '{{}}' are required arguments "
                "and values between brackets '()' are optional arguments. If "
                "don't want optional arguments, left them blank. In case you do"
                " not specify a username and password, you must add the '-a' "
                "option which specifies that the connection is made with the "
                "anonymous user.{0}{0}The order of arguments doesn't matter, "
                "but all are needed.{0}{0}").format(os.linesep)

    parser = ArgumentParser(description=main_desc, epilog=epilog)
    subparsers = parser.add_subparsers(title="running modes",
                                       description=subs_desc)

    cron = subparsers.add_parser("cron", help="ideal to run as a scheduled "
                                              "task. Takes arguments from parameters "
                                              "defined within the script", usage=cron_use)
    cron.add_argument("cron", action="store_true", help=SUPPRESS,
                      default=SUPPRESS)
    cron.add_argument("cfg", action="store_false", help=SUPPRESS,
                      default=SUPPRESS)
    cron.add_argument("config_file_path", help="tuner config file to import arguments")
    cron.add_argument("audio_dir", help="tuner audio files path")
    cron.add_argument("logs_dir", help="tuner logs dir")

    cfg = subparsers.add_parser("cfg", help="ideal for mirror multiple sites/"
                                            "directories. Imports the arguments from a "
                                            "config file", usage=file_use)
    cfg.add_argument("cron", action="store_false", help=SUPPRESS,
                     default=SUPPRESS)
    cfg.add_argument("cfg", action="store_true", help=SUPPRESS,
                     default=SUPPRESS)
    cfg.add_argument("config_file", help="config file to import arguments")

    shell = subparsers.add_parser("shell", help="usual mode, takes arguments "
                                                "from the command line ", usage=shell_use)
    shell.add_argument("cron", action="store_false", help=SUPPRESS,
                       default=SUPPRESS)
    shell.add_argument("cfg", action="store_false", help=SUPPRESS,
                       default=SUPPRESS)
    shell.add_argument("site", help="the ftp server (URL or IP)")
    shell.add_argument("remote", help="the remote directory")
    shell.add_argument("local", help="the local directory")

    auth = shell.add_mutually_exclusive_group(required=True)
    auth.add_argument("-l", "--login", dest="login", nargs=2,
                      help="the ftp account's username and password",
                      metavar=("user", "password"))
    auth.add_argument("-a", "--anon", action="store_true", dest="anonymous",
                      help="set user as anonymous", default=False)

    shell.add_argument("-p", "--port", dest="port", default="",
                       help="the ftp server port", metavar="port")
    shell.add_argument("-s", "--secure", action="store_const", const="s",
                       dest="secure", default="",
                       help="use the sftp protocol instead of ftp")
    shell.add_argument("-e", "--erase", action="store_const", const="e",
                       dest="erase", default="",
                       help="delete files not present at target site")
    shell.add_argument("-n", "--newer", action="store_const", const="n",
                       dest="newer", default="",
                       help="download only newer files")
    shell.add_argument("-P", "--parallel", dest="parallel", default="",
                       nargs="?", metavar="N", const=2,
                       help="download N files in parallel. N=2 if not provide "
                            "any value")
    shell.add_argument("-r", "--reverse", action="store_const", const="R",
                       dest="reverse", default="",
                       help="reverse, upload files from local to remote")
    shell.add_argument("--delete-first", action="store_const",
                       const=" --delete-first", dest="del_first", default="",
                       help="delete old files before transferring new ones")
    shell.add_argument("--depth-first", action="store_const",
                       const=" --depth-first", dest="depth_first", default="",
                       help="descend into subdirectories, before transfer "
                            "files")
    shell.add_argument("--no-empty-dirs", action="store_const",
                       const=" --no-empty-dirs", dest="no_empty_dir",
                       default="",
                       help="don't create empty dirs (needs --depth-first)")
    shell.add_argument("--no-recursion", action="store_const",
                       const=" --no-recursion", dest="no_recursion",
                       default="",
                       help="don't go to subdirectories")
    shell.add_argument("--dry-run", action="store_const",
                       const=" --dry-run", dest="dry_run", default="",
                       help="simulation, don't execute anything. Writes to "
                            "log")
    shell.add_argument("--use-cache", action="store_const",
                       const=" --use-cache", dest="use_cache", default="",
                       help="use cached directory listings")
    shell.add_argument("--del-source", action="store_const",
                       const=" --Remove-source-files",
                       dest="del_source", default="",
                       help="remove files (no dirs) after transfer (Caution!)")
    shell.add_argument("--only-missing", action="store_const",
                       const=" --only-missing", dest="missing", default="",
                       help="download only missing files")
    shell.add_argument("--only-existing", action="store_const",
                       const=" --only-existing", dest="existing", default="",
                       help="download only files already existing at target")
    shell.add_argument("--loop", action="store_const",
                       const=" --loop", dest="loop", default="",
                       help="loop until no changes found")
    shell.add_argument("--ignore-size", action="store_const",
                       const=" --ignore-size", dest="size", default="",
                       help="ignore size when deciding whether to download")
    shell.add_argument("--ignore-time", action="store_const",
                       const=" --ignore-time", dest="time", default="",
                       help="ignore time when deciding whether to download")
    shell.add_argument("--no-perms", action="store_const",
                       const=" --no-perms", dest="no_perms", default="",
                       help="don't set file permissions")
    shell.add_argument("--no-umask", action="store_const",
                       const=" --no-umask", dest="no_umask", default="",
                       help="don't apply umask to file modes")
    shell.add_argument("--no-symlinks", action="store_const",
                       const=" --no-symlinks", dest="no_symlinks", default="",
                       help="don't create symbolic links")
    shell.add_argument("--allow-suid", action="store_const",
                       const=" --allow-suid", dest="suid", default="",
                       help="set suid/sgid bits according to remote site")
    shell.add_argument("--allow-chown", action="store_const",
                       const=" --allow-chown",
                       dest="chown", default="",
                       help="try to set owner and group on files")
    shell.add_argument("--dereference", action="store_const",
                       const=" --dereference", dest="dereference", default="",
                       help="download symbolic links as files")
    shell.add_argument("--exclude-glob", action="append", dest="exc_glob",
                       default=[], metavar="GP",
                       help="exclude matching files. GP is a glob pattern, "
                            "e.g. '*.zip'")
    shell.add_argument("--include-glob", action="append", dest="inc_glob",
                       default=[], metavar="GP",
                       help="include matching files. GP is a glob pattern, "
                            "e.g. '*.zip'")

    shell.add_argument("-q", "--quiet", action="store_true", dest="quiet",
                       help="the detailed shell process is no "
                            "displayed, but is added to the log", default=False)
    shell.add_argument("--no-compress", action="store_true",
                       dest="no_compress", help="don't create daily archive "
                                                "files", default=False)
    shell.add_argument("--no-email", action="store_true", dest="no_email",
                       help="no sends email with the log", default=False)
    shell.add_argument("--smtp_server", dest="smtp_server",
                       default="localhost", metavar="server",
                       help="set a smtp server")
    shell.add_argument("--smtp_user", dest="smtp_user", default="",
                       metavar="user", help="the smtp server username")
    shell.add_argument("--smtp_pass", dest="smtp_pass", default="",
                       metavar="password", help="the smtp server password")
    shell.add_argument("--from_addr", dest="from_addr", default="",
                       metavar="email", help="sender's email address")
    shell.add_argument("--to_addrs", dest="to_addrs", default="", nargs='+',
                       metavar="email",
                       help="a list of receiver(s)' email address(es)")

    parser.add_argument("-v", "--version", action="version",
                        version="%(prog)s {0}".format(__version__),
                        help="show program's version number and exit")
    return parser


def check_execs_posix_win(*progs):
    """Check if the programs are installed.

    Returns two values:

    (dict) windows_paths - a dictionary of executables/paths (keys/values)
    (boolean) is_windows - True it's a Windows OS, False it's a *nix OS

    """

    def not_found(app):
        """ If executable is not installed, exit and report."""
        msg = 'The {0} program is necessary to run this script'.format(app)
        sys.exit(msg)

    windows_paths = {}
    is_windows = True if platform.system() == 'Windows' else False
    # get all the drive unit letters if the OS is Windows
    windows_drives = re.findall(r'(\w:)\\',
                                Popen('fsutil fsinfo drives', stdout=PIPE).
                                communicate()[0]) if is_windows else None
    for prog in progs:
        if is_windows:
            # Set all commands to search the executable in all drives
            win_cmds = ['dir /B /S {0}\*{1}.exe'.format(letter, prog) for
                        letter in windows_drives]
            # Get the first location (usually in C:) of the all founded where
            # the executable exists
            exe_paths = (''.join([Popen(cmd, stdout=PIPE, stderr=PIPE,
                                        shell=True).communicate()[0] for
                                  cmd in win_cmds])).split(os.linesep)[0]
            # Assign the path to the executable or report not found if empty
            windows_paths[prog] = exe_paths if exe_paths else not_found(prog)
        else:
            try:
                Popen([prog, '--help'], stdout=PIPE, stderr=PIPE)
            except OSError:
                not_found(prog)
    return windows_paths, is_windows


def best_unit_size(bytes_size):
    """Get a size in bytes & convert it to the best IEC prefix for readability.

    Return a dictionary with three pair of keys/values:

    's' -- (float) Size of path converted to the best unit for easy read
    'u' -- (str) The prefix (IEC) for s (from bytes(2^0) to YiB(2^80))
    'b' -- (int / long) The original size in bytes

    """
    for exp in range(0, 90, 10):
        bu_size = abs(bytes_size) / pow(2.0, exp)
        if int(bu_size) < 2 ** 10:
            unit = {0: 'bytes', 10: 'KiB', 20: 'MiB', 30: 'GiB', 40: 'TiB',
                    50: 'PiB', 60: 'EiB', 70: 'ZiB', 80: 'YiB'}[exp]
            break
    return {'s': bu_size, 'u': unit, 'b': bytes_size}


def get_size(the_path):
    """Get size of a directory tree or a file in bytes."""
    path_size = 0
    for path, directories, files in os.walk(the_path):
        for filename in files:
            path_size += os.lstat(os.path.join(path, filename)).st_size
        for directory in directories:
            path_size += os.lstat(os.path.join(path, directory)).st_size
    path_size += os.path.getsize(the_path)
    return path_size


def mirror(args, tuner_id):
    """Mirror the directories."""

    user = '' if args.anonymous else ' '.join(args.login)
    local, remote = os.path.normpath(args.local), os.path.normpath(args.remote)
    port = '-p {0}'.format(args.port) if args.port else ''
    include = ''
    for iglob in args.inc_glob:
        include += ' --include-glob {0}'.format(iglob)
    exclude = ''
    for eglob in args.exc_glob:
        exclude += ' --exclude-glob {0}'.format(eglob)
    parallel = ' --parallel={0}'.format(args.parallel) if args.parallel else ''

    msg = 'Connected to {0} as {1}'.format(args.site,
                                           'anonymous' if args.anonymous
                                           else args.login[0])
    logging.info(msg)

    msg = 'Mirror {0} to {1}'.format(local if args.reverse else remote,
                                     remote if args.reverse else local)
    logging.info(msg)

    if not os.path.exists(local):
        os.mkdir(local)
    os.chdir(os.path.join(local, os.pardir))

    # create the script file to import with lftp
    scp_args = ('-vvv' + args.erase + args.newer + args.reverse
                + args.del_first + args.depth_first + args.no_empty_dir +
                args.no_recursion + args.dry_run + args.use_cache +
                args.del_source + args.missing + args.existing + args.loop +
                args.size + args.time + args.no_perms + args.no_umask +
                args.no_symlinks + args.suid + args.chown + args.dereference +
                exclude + include + parallel)

    ftpscript = os.path.join(get_ftp_status_dir(tuner_id), 'ftpscript')
    with open(ftpscript, 'w') as script:
        lines = ('open {0}ftp://{1} {2}'.format(args.secure, args.site, port),
                 'user {0}'.format(user),
                 'mirror {0} {1} {2}'.format(scp_args,
                                             local if args.reverse else remote,
                                             remote if args.reverse else local),
                 'exit')
        script.write(os.linesep.join(lines))

    msg = u'[info]\tИдет загрузка ({0})'.format(time.strftime("%Y-%m-%d %H:%M:%S"))
    write_status(tuner_id, msg)

    # mirror
    cmd = ['lftp', '-d', '-f', script.name]
    sync = Popen(cmd, stdout=PIPE, stderr=STDOUT, bufsize=1)
    # end mirroring

    file_size = 0
    file_upload_start = datetime.datetime.now()
    some_error = False
    transfer_started = False
    for line in iter(sync.stdout.readline, b''):
        msg = line.strip('\n')
        if '<---' in line:
            try:
                msg = ftp_error_codes[int(msg[5:8])]
                line = line.strip('\n')[9:]
            except:
                pass

        if 'Peer closed connection' in line:
            msg = u'Сервер закрыл соединение'
            line = line.strip('\n')
        if 'Timeout - reconnecting' in line:
            msg = u'Время ожидания подключения истекло'
            line = line.strip('\n')
        if 'Name or service not known' in line:
            msg = u'Не удалось подключиться к {0}'.format(args.site)
            line = line.strip('\n')
        if 'Socket error (Connection refused)' in line:
            msg = u'{0} отклонил подключение'.format(args.site)
            line = line.strip('\n')
        if 'Socket error (No route to host)' in line:
            msg = u'Нет маршрута к хосту {0}'.format(args.site)
            line = line.strip('\n')
        if 'Socket error (Connection timed out)' in line:
            msg = u'Время ожидания подключения к хосту {0} истекло'.format(args.site)
            line = line.strip('\n')

        if msg != line.strip('\n'):
            some_error = True
            dt = u' ({0})'.format(time.strftime("%Y-%m-%d %H:%M:%S"))
            logging.error(line)
            write_status(tuner_id, msg + dt)
            sync.kill()
            break

        if 'Transferring file' in line:
            transfer_started = True
            line = line.strip('\n')
            file_path = line[line.index('file ')+6:].strip('\'')
            file_path = os.path.join(local if args.reverse else remote, file_path)
            file_size = get_size(file_path) /float(1024) / float(1024)
            file_upload_start = datetime.datetime.now()
            line = line.strip('\n')

        if '<--- 226' in line and transfer_started:
            now = datetime.datetime.now()
            upload_time = (now - file_upload_start).total_seconds()
            upload_speed = file_size / float(upload_time)
            logging.info('File successfully transferred with speed {0:0.2f} mb/s for {1:0.2f} seconds'.format(upload_speed, upload_time))
            transfer_started = False
            continue

        if 'is not removed' not in line and '---' not in line and '****' not in line:
            logging.info(line.strip('\n'))

    if not some_error:
        local_size = get_size(local)
        size = best_unit_size(local_size)
        msg = u'[info]\tЗагружено ({0})'.format(time.strftime("%Y-%m-%d %H:%M:%S"))
        write_status(tuner_id, msg)
        msg = 'Local disk space used {0} {1}'.format(int(size['s']), size['u'])
        logging.info(msg)

    try:
        os.remove(script.name)
    except OSError:
        pass


def parse_parms(*parms):
    """Parse parameters from script or config file to shell format."""
    parameters = ("shell {0} {1} {2} {3} {4} {5} {6} {7}".
                  format(parms[0],
                         '-p {0}'.format(parms[1]) if parms[1] else '',
                         parms[2],
                         parms[3],
                         '-l {0}'.format(parms[4]) if parms[4] else '',
                         parms[5],
                         parms[6],
                         '-s' if parms[7] else ''))
    return parameters.split()


def main():
    # first, parse the arguments
    args = arguments().parse_args()

    # set the arguments depending of execution mode
    if args.cron:
        parser = config_utils.get_parser(args.config_file_path)
        sec = 'global'
        # ftp user name ('user' by default)
        cron_user = parser.get(sec, 'ftp-user')
        # ftp password ('password' by default)
        cron_pass = parser.get(sec, 'ftp-pwd')
        # ftp server, name or ip ('localhost' by default)
        cron_site = parser.get(sec, 'ftp-url')
        m = re.search('(ftp|sftp)://', cron_site)
        cron_protocol = m.group(1)
        cron_secure = (cron_protocol == 'sftp')
        cron_site = cron_site[cron_site.index("://") + 3:]
        # ftp server port. ('' by default)
        cron_port = ''
        if ':' in cron_site:
            m = re.search('(\w*):(\d*)', cron_site)
            cron_port = m.group(2)
        # ftp directory
        cron_remote = '/'
        if '/' in cron_site:
            cron_remote = cron_site[cron_site.index('/'):]

        cron_site = cron_site[:cron_site.index('/')]
        # local directory
        cron_local = args.audio_dir
        # options, same as the shell mode. See shell mode help for more info cron_options =
        cron_options = '-r -n --ignore-size --exclude-glob *.json'
        if parser.getboolean(sec, 'ftp-metadata'):
            cron_options = '-r -n --ignore-size'

        logs_dir = args.logs_dir
        logfilename = create_logfile(logs_dir)

        args = arguments().parse_args(parse_parms(cron_site,
                                                  cron_port,
                                                  cron_remote,
                                                  cron_local,
                                                  cron_user,
                                                  cron_pass,
                                                  cron_options,
                                                  cron_secure))

        logging.basicConfig(filename=logfilename, level=logging.DEBUG, format='%(asctime)s\t%(levelname)s\t%(message)s')

        tuner_id = logs_dir.split('/')[4]
        mirror_as_single(args, tuner_id)

    elif args.cfg:
        cfg = SafeConfigParser()
        cfg.read(args.config_file)
        for sect in cfg.sections():
            args = arguments().parse_args(parse_parms(cfg.get(sect, 'site'),
                                                      cfg.get(sect, 'port'),
                                                      cfg.get(sect, 'remote'),
                                                      cfg.get(sect, 'local'),
                                                      cfg.get(sect, 'user'),
                                                      cfg.get(sect, 'password'),
                                                      cfg.get(sect, 'options')))
            mirror(args, '')
    else:
        mirror(args, '')


def create_logfile(logs_dir):
    datestr = time.strftime('%Y/%m/%d/')
    timestr = time.strftime('%H%M%S')
    filename = os.path.join(logs_dir, '{0}lftp_mirror_{1}.log'.format(datestr, timestr))
    dirs = os.path.join(logs_dir, datestr)
    if not os.path.exists(dirs):
        os.makedirs(dirs)
    return filename


def write_status(tuner_id, msg):
    status_dir = get_ftp_status_dir(tuner_id)
    if not os.path.exists(status_dir):
        os.makedirs(status_dir)
    with open(get_ftp_status_file_path(tuner_id), 'w') as status:
        status.write(msg.encode('utf-8'))
        status.close()


def update_status(tuner_id):
    msg = u'[warning]\tЗагрузка уже осуществляется, запуск прерван. Попытка №'.encode('utf-8')
    curtime = u'\n({0})'.format(time.strftime("%Y-%m-%d %H:%M:%S")).encode('utf-8')
    with open(get_ftp_status_file_path(tuner_id), 'r') as status:
        mirror_status = status.readline()
        if 'Идет загрузка' in mirror_status:
            msg += str(1)
            write_status(tuner_id, (msg + curtime).decode('utf-8'))
        if msg in mirror_status:
            tries_cnt = int(mirror_status[mirror_status.index('№') + 3:mirror_status.index('\n')])
            msg += str(tries_cnt + 1)
            write_status(tuner_id, (msg + curtime).decode('utf-8'))


def get_ftp_status_file_path(tuner_id):
    return os.path.join('/tmp/tapperui', tuner_id, 'ftp_service_status')


def get_ftp_status_dir(tuner_id):
    return os.path.join('/tmp/tapperui', tuner_id)


def get_ftp_status(tuner_id):
    status_file = get_ftp_status_file_path(tuner_id)
    if os.path.isfile(status_file):
        with open(status_file, 'r') as status:
            return status.read()
    else:
        return ''


def mirror_as_single(args, tuner_id):
    pid = str(os.getpid())

    pid_dir = get_ftp_status_dir(tuner_id)
    if not os.path.exists(pid_dir):
        os.makedirs(pid_dir)

    pidfile = os.path.join(pid_dir, 'lftp_mirror.pid')

    if os.path.isfile(pidfile):
        msg = u'Загрузка уже осуществляется, выходим...'
        logging.warning(msg)
        update_status(tuner_id)
        print(msg)
        sys.exit()
    else:
        file(pidfile, 'w').write(pid)

    try:
        mirror(args, tuner_id)
    except Exception, e:
        logging.exception(e)
        write_status(tuner_id, str(e))

    os.unlink(pidfile)


if __name__ == "__main__":
    check_execs_posix_win('lftp')  # Check first if lftp is installed
    main()
