# -*- coding: utf-8 -*-
import errno
import fmtapper_web.errors

import os
from flask import current_app, send_file, Response, request
import subprocess

import psutil
from json_utils import JSONIterator
from utils import parse_audio_devices_output, query_db
from tuner import Tuner, RuntimeInfo
import config_utils
from errors import ServiceMethodError, ConfigError
import shutil
from schedule import Schedule, Schedules
import tuner_service_utils
import runtime_utils
from zipfile import ZipFile
from cleanup_service import CleanupService
from crontab import CronTab
import re
import zipstream
import zipfile
import mimetypes
import re


class SystemInfo(JSONIterator):

    def __init__(self, partitions, metrics):
        self.partitions = partitions
        self.metrics = metrics

    def to_json(self):
        rv = dict()
        rv['metrics'] = self.metrics
        rv['partitions'] = self.partitions
        return rv


class Partition(JSONIterator):

    def __init__(self, mountpoint, dev, usage):
        self.mountpoint = mountpoint
        self.dev = dev
        self.usage = usage

    def __str__(self):
        return "Partition(mountpoint=%s, device=%s, %s)" % (self.mountpoint, self.dev, self.usage)

    def to_json(self):
        rv = dict()
        rv['mountpoint'] = self.mountpoint
        rv['dev'] = self.dev
        rv['usage'] = self.usage
        return rv


def get_system_info(db):
    def get_partition_usage(mountpoint):
        rv = dict()
        usage = psutil.disk_usage(mountpoint)
        rv['total'] = usage[0]
        rv['used'] = usage[1]
        rv['free'] = usage[2]
        rv['percent'] = usage[3]
        return rv

    partitions = map(lambda p: Partition(mountpoint=p.mountpoint, dev=p.device,
                                         usage=get_partition_usage(p.mountpoint)), psutil.disk_partitions())
    query = 'select * from metrics where ts > strftime(\'%s\', \'now\') - {} order by ts'\
        .format(current_app.config['METRICS_LIFETIME_SEC'])
    metrics = query_db(db, query)
    return SystemInfo(partitions=partitions, metrics=metrics)


def get_tuners():
    current_app.logger.debug(
        "Getting tuner's configuration from base dir %s", config_utils.get_profiles_dir())
    tuners = []
    for t_id in get_tuner_ids():
        try:
            tuners.append(get_tuner(t_id))
        except Exception, e:
            current_app.logger.error(
                "Error while reading tuner configuration", exc_info=e)
    return tuners


def get_tuner_ids():
    config_dir = config_utils.get_profiles_dir()
    ids = []
    for (dirpath, dirnames, filenamses) in os.walk(config_dir):
        ids.extend(dirnames)
        break
    return ids


def get_tuner(tuner_id):
    logger = current_app.logger
    logger.debug('Getting configuration for tuner %s', tuner_id)
    t = Tuner.load_from_file(tuner_id)
    t.log_dir = config_utils.get_tuner_logs_dir(tuner_id)
    t.runtime_info = get_tuner_status(tuner_id)
    return t


def apply_tuner(tuner):
    current_app.logger.debug('Apllying configuration for tuner %s', tuner.id)
    apply_schedules(tuner.id)
    tuner_service_utils.restart_tuner(tuner.id)
    tuner.is_changes_applied = True
    config_dir = config_utils.get_profiles_dir()
    file_path = config_utils.get_config_file_path(tuner.id) + '.new'
    if os.path.isfile(file_path):
        current_app.logger.debug(
            'Removing temporary configuration file %s for tuner %s', file_path, tuner.id)
        os.remove(file_path)
    tuner.save_to_file(config_dir)


def apply_schedules(tuner_id):
    schedules_list = Schedules.get_schedules_list(tuner_id)
    schedules_file = config_utils.get_schedules_applied_file_path(tuner_id)
    if not os.path.exists(schedules_file[:-17]):
        os.makedirs(schedules_file[:-17])
    with open(schedules_file, 'w') as fp:
        for sched in schedules_list:
            fp.write(sched + '\n')


def apply_schedule(schedule, tuner_id):
    current_app.logger.debug('Apllying new schedules for tuner %s', tuner_id)
    tuner_service_utils.restart_tuner(tuner_id)
    schedule.is_changes_applied = True
    schedule.set_applied_file(tuner_id)
    config_dir = config_utils.get_schedules_dir(tuner_id)
    schedule.save_to_file(config_dir, tuner_id)

def update_tuner_udp_port(tuner_id, start=True):
    tuner = Tuner.load_from_file(tuner_id)
    if tuner.device_type != Tuner.DeviceType.FM_TUNER:
        return True

    if start:
        free_udp_port = config_utils.get_free_udp_port()
        if free_udp_port == 0:
            return False
        tuner.device_config.udp_port = free_udp_port
        config_utils.update_ports_in_use(free_udp_port, start=start)
    else:
        config_utils.update_ports_in_use(tuner.device_config.udp_port, start=start)
        tuner.device_config.udp_port = 0
    tuner.save_to_file()

    return True

def update_tuner(tuner):
    tuner_old = get_tuner(tuner.id)
    current_app.logger.debug('Saving configuration for tuner %s', tuner.id)
    config_dir = config_utils.get_profiles_dir()
#   create logging config
    tuner_service_utils.gen_logging_config(tuner.id)

    if get_tuner_status(tuner.id).status == 0:
        tuner.set_restart(tuner_old)
    else:
        tuner.is_changes_applied = True

    # if tuner_old.device_type == Tuner.DeviceType.FM_TUNER or \
    #                 tuner.device_type == Tuner.DeviceType.FM_TUNER:
    #     update_ports_in_use(tuner_old, tuner)

    tuner.save_to_file(config_dir)
    tuner_service_utils.gen_init_script(
        tuner, config_utils.get_config_file_path(tuner.id))
    # update clean up service
    cleanup_service = get_clenup_service()
    update_cleanup_service(cleanup_service)
    # update ftp service
    logs_dir = config_utils.get_tuner_logs_dir(tuner.id)
    update_ftp_service(config_utils.get_config_file_path(tuner.id), logs_dir)

def update_cron_job(job, minutes, hours, days):
    if minutes != 0:
        job.minute.every(minutes)
    if hours != 0:
        job.hour.every(hours)
        job.minute.on(0)
    if days != 0:
        job.day.every(days)
        job.hour.on(0)
        job.minute.on(0)

def update_ftp_service(config_file_path, logs_dir, delete=False):
    parser = config_utils.get_parser(config_file_path)

    audio_dir = parser.get('global', 'url')
    ch_id = parser.get('global', 'channel-id')
    audio_dir = os.path.join(audio_dir[:audio_dir.index("{")], ch_id)

    cmd = 'python /opt/tapperui/fmtapper_web/lftp_mirror.py cron {0} {1} {2}'.format(
        config_file_path, audio_dir, logs_dir)

    system_cron = CronTab(tabfile='/etc/crontab', user=False)
    cron_jobs = system_cron.find_command(cmd)
    for job in cron_jobs:
        system_cron.remove(job)

    if delete:
        ftp_url = ''
    else:
        ftp_url = parser.get('global', 'ftp-url')

    if ftp_url != '':
        cron_time = parser.get('global', 'ftp-upload-interval')
        m = re.search('\A(\d\d):(\d\d):(\d\d)\Z', cron_time)
        days = int(m.group(1))
        hours = int(m.group(2))
        minutes = int(m.group(3))

        cron_job = system_cron.new(command=cmd, user='root')
        update_cron_job(cron_job, minutes, hours, days)

    system_cron.write()


def create_tuner(tuner, copy_schedules):
    current_app.logger.debug('Creating tuner profile %s', tuner.id)
    config_dir = config_utils.get_profiles_dir()
    tuner.save_to_file(config_dir)
    # if tuner.device_type == Tuner.DeviceType.FM_TUNER:
    #     update_ports_in_use(tuner=tuner)

    if copy_schedules:
        new_tuner_sched_dir = os.path.join(config_utils.get_new_tuner_tmp_dir(),
                                           config_utils.get_schedules_dir_name())
        shutil.copytree(new_tuner_sched_dir,
                        config_utils.get_schedules_dir(tuner.id))

#   create logging config
    tuner_service_utils.gen_logging_config(tuner.id)
#   create system service
    tuner_service_utils.gen_init_script(
        tuner, config_utils.get_config_file_path(tuner.id))
    returncode = tuner_service_utils.start_tuner(tuner.id)
    if returncode == 0:
        update_tuner_udp_port(tuner.id, start=True)

    # update clean up service
    cleanup_service = get_clenup_service()
    update_cleanup_service(cleanup_service)
    # update ftp service
    logs_dir = config_utils.get_tuner_logs_dir(tuner.id)
    update_ftp_service(config_utils.get_config_file_path(tuner.id), logs_dir)


def delete_tuner(tuner_id):
    logger = current_app.logger
    logger.debug("Deleting tuner %s", tuner_id)
    tuner_dir = os.path.join(config_utils.get_profiles_dir(), tuner_id)
    if os.path.isdir(tuner_dir):
        tuner = get_tuner(tuner_id)
        # if tuner.device_type == Tuner.DeviceType.FM_TUNER:
        #     update_ports_in_use(tuner_old=tuner)

        returncode = tuner_service_utils.stop_tuner(tuner_id)
        if returncode == 0:
            update_tuner_udp_port(tuner_id, start=False)
        tuner_service_utils.del_init_script(tuner_id)

        logs_dir = config_utils.get_tuner_logs_dir(tuner_id)
        if os.path.isdir(logs_dir):
            logger.debug('Deleting logs dir %s', logs_dir)
            shutil.rmtree(logs_dir)
        else:
            logger.debug("Logs dir %s doesn't exist", logs_dir)

        audio_dir = get_tuner_audio_dir(tuner_id)
        if os.path.isdir(audio_dir):
            try:
                logger.debug('Deleting media dir %s',
                             os.path.join(audio_dir, tuner_id))
                shutil.rmtree(os.path.join(audio_dir, tuner_id))
                logger.debug('Deleting media dir %s', audio_dir)
                os.rmdir(audio_dir)
            except OSError as ex:
                if ex.errno == errno.ENOTEMPTY:
                    logger.debug('Media dir %s is not empty', audio_dir)
                if ex.errno == errno.ENOENT:
                    logger.debug('No such file or directory: %s',
                                 os.path.join(audio_dir, tuner_id))
        else:
            logger.debug("Media dir %s does't exist", audio_dir)

        state_file = tuner_service_utils.get_state_file_path(
            tuner_id, config_utils.get_state_dir())
        if os.path.isfile(state_file):
            logger.debug("Deleting state file %s", state_file)
            os.remove(state_file)
        else:
            logger.debug("State file %s doesn't exist", state_file)

        logs_dir = config_utils.get_tuner_logs_dir(tuner_id)
        update_ftp_service(config_utils.get_config_file_path(
            tuner_id), logs_dir, delete=True)

        logger.debug("Deleting %s", tuner_dir)
        shutil.rmtree(tuner_dir)

        tmp_dir = config_utils.get_tmp_dir(tuner_id)
        if os.path.exists(tmp_dir):
            logger.debug("Deleting %s", tmp_dir)
            shutil.rmtree(tmp_dir)

        # update clean up service
        cleanup_service = get_clenup_service()
        update_cleanup_service(cleanup_service)
    else:
        logger.error(
            "Tuner's dir is not directory or not exists, %s", tuner_dir)
        raise ServiceMethodError("Can't delete")


def get_audio_devices():
    current_app.logger.debug('Getting list of available audio devices')
    p = subprocess.check_output(['/usr/bin/fmtapper', '--list-devices'])
    devices = parse_audio_devices_output(p.split('\n'))

    tuners = get_tuners()
    for device in devices:
        device_tuners = filter(lambda t: not t.device_type == 2 and not t.device_type == 3 and t.device_config
                               and device.name == t.device_config.name, tuners)
        if device_tuners:
            if len(device_tuners) > 1:
                raise fmtapper_web.errors.ConfigError('Device with name %s used in more than 1 tuner profile' %
                                                      device.name)
            else:
                device.tuner = device_tuners[0].id
    return devices


def get_schedules(tuner_id):
    """Возвращает объект раписаний.

        Возврашает объект расписаний на основе файлов с раширением .sch в папке расписаний.
        :param tuner_id Id профиля тюнера

    """
    logger = current_app.logger
    sch_dir = config_utils.get_schedules_dir(tuner_id)

    default_sch = False
    week_sch = []
    date_sch = []

    if os.path.exists(sch_dir):
        if not os.path.isdir(sch_dir):
            msg = 'Schedules directory is not directory ' + sch_dir
            logger.error(msg)
            raise fmtapper_web.errors.ConfigError(msg)
        else:
            names = []
            for (dirpath, dirnames, filenames) in os.walk(sch_dir):
                names.extend(
                    map(lambda f: f[0:-4], filter(lambda f: f.endswith('.sch'), filenames)))
                break
            default_sch = Schedules.SCHEDULE_DEFAULT_NAME in names
            week_sch = filter(
                lambda s: s in Schedules.SCHEDULE_WEEK_NAMES, names)
            date_sch = filter(lambda s: s not in Schedules.SCHEDULE_WEEK_NAMES
                              and not s == Schedules.SCHEDULE_DEFAULT_NAME, names)

    return Schedules(default_sch=default_sch, week_sch=week_sch, date_sch=date_sch, tuner_id=tuner_id)


def get_schedule(tuner_id, sch_id):
    """ Возвращает расписание

    :param tuner_id Id профиля тюнера
    :param sch_id: имя расписания
    :return: расписание
    """
    logger = current_app.logger
    logger.debug('Get schedule %s', sch_id)
    file_name = sch_id + '.sch'
    file_path = os.path.join(
        config_utils.get_schedules_dir(tuner_id), file_name)
    if not os.path.isfile(file_path):
        raise ServiceMethodError('Schedule %s does not exist' % file_path)
    return Schedule.load_from_file(file_path, tuner_id)


def update_schedule(tuner_id, schedule):
    current_app.logger.debug(
        'Updating schedule %s for tuner %s', schedule.id, tuner_id)

    if get_tuner_status(tuner_id).status == 0 and get_tuner(tuner_id).schedule:
        old_schedule = get_schedule(tuner_id, schedule.id)
        schedule.set_restart(old_schedule)
    else:
        schedule.is_changes_applied = True

    schedule.save_to_file(config_utils.get_schedules_dir(tuner_id), tuner_id)


def create_schedule(tuner_id, schedule):
    logger = current_app.logger
    logger.debug('Creating schedule %s for tuner %s', schedule.id, tuner_id)
    dir_path = config_utils.get_schedules_dir(tuner_id)
    if not os.path.exists(dir_path):
        logger.debug("Creating schedule dir %s", dir_path)
    elif not os.path.isdir(dir_path):
        raise ConfigError("Schedule dir is not dir %s" % dir_path)

    if get_tuner_status(tuner_id).status != 0 or not get_tuner(tuner_id).schedule:
        schedule.is_changes_applied = True
        schedule.set_applied_file(tuner_id)

    schedule.save_to_file(dir_path, tuner_id)


def delete_schedule(tuner_id, sch_id):
    logger = current_app.logger
    logger.debug('Deleting schedule %s for tuner %s', sch_id, tuner_id)
    logger.debug('get_tuner_status %s', get_tuner_status(tuner_id).status)

    if get_tuner_status(tuner_id).status == Tuner.TunerStatus.STOP or not get_tuner(tuner_id).schedule:
        schedule = get_schedule(tuner_id, sch_id)
        schedule.is_changes_applied = False
        schedule.set_applied_file(tuner_id)

    file_path = os.path.join(
        config_utils.get_schedules_dir(tuner_id), sch_id + '.sch')
    if os.path.isfile(file_path):
        os.remove(file_path)
    else:
        raise ServiceMethodError('Schedule %s does not exist', file_path)


def get_audio_files(tuner_id):
    """
    Возвращает список объектов FileInfo для всех файлов записи тюнера
    :param tuner_id: Id тюнера
    :return: список FileInfo
    """
    logger = current_app.logger
    logger.debug("Get audio files for tuner %s", tuner_id)
    basedir = get_tuner_audio_dir(tuner_id)
    logger.debug("Media directory is %s", basedir)
    return _get_files(basedir)


def get_log_files(tuner_id):
    """
    Возвращает список объектов FileInfo для всех файлов логов тюнера
    :param tuner_id: Id тюнера
    :return: список FileInfo
    """
    logger = current_app.logger
    logger.debug("Get log files for tuner %s", tuner_id)
    logs_dir = config_utils.get_tuner_logs_dir(tuner_id)
    if not os.path.isdir(logs_dir):
        logger.warn("Log directory does't exist for tuner %s", tuner_id)
        return []
    else:
        return _get_files(logs_dir)


def _get_files(basedir, recursive=True):
    parent = _get_dir_files(basedir, basedir, recursive)
    parent.path = ""
    parent.name = ""
    parent.sort()
    return parent


def _get_dir_files(parent_dir, basedir, recursive):
    children = []
    for (dirpath, dirnames, filenames) in os.walk(parent_dir):
        if recursive:
            for dirname in dirnames:
                children.append(_get_dir_files(os.path.join(
                    dirpath, dirname), basedir, recursive))

        for filename in filenames:
            file_path = os.path.join(dirpath, filename)
            (mode, ino, dev, nlink, uid, gid, size,
             atime, mtime, ctime) = os.stat(file_path)
            children.append(FileInfo(name=filename, path=_extract_basedir(
                basedir, file_path), size=size, mtime=mtime))

        break

    return FileInfo(name=os.path.basename(parent_dir), path=_extract_basedir(basedir, parent_dir), size=0, mtime=None,
                    children=children)


def _extract_basedir(basedir, path):
    res = path.replace(basedir, '', 1)
    if res.startswith('/'):
        res = res[1:]
    return res


def get_tuner_audio_dir(tuner_id):
    """
    Возвращает путь к папке файлов записей тюнера
    :rtype : str
    :param tuner_id: Id тюнера
    :return: путь
    """
    t = get_tuner(tuner_id)
    if t is None:
        raise ServiceMethodError("Tuner %s not exists" % tuner_id)
    if t.encoding_config is None or t.encoding_config.url is None:
        raise ServiceMethodError(
            "Audio directory for tuner %s is not configured" % tuner_id)

    return t.get_audio_dir()


def get_installation_id(file_path):
    if os.path.isfile(file_path):
        with open(file_path, 'r') as f:
            return f.readline()
    else:
        raise ConfigError("Installation Id file doesn't exist: " + file_path)


class FileInfo(JSONIterator):

    def __init__(self, name, path, mtime, size, children=None):
        self.name = name
        self.path = path
        self.mtime = mtime
        self.size = size
        self.children = children

    def to_json(self):
        rv = dict()
        rv['name'] = self.name
        rv['path'] = self.path
        rv['size'] = self.size
        rv['mtime'] = self.mtime
        rv['children'] = self.children
        return rv

    def sort(self):
        if self.children is not None:
            self.children.sort()
            for child in self.children:
                child.sort()

    def __cmp__(self, o):
        if self.children is not None:
            if o.children is not None:
                return FileInfo._strcmp(self.name, o.name)
            else:
                return -1
        else:
            if o.children is not None:
                return 1
            else:
                return FileInfo._strcmp(self.name, o.name)

    @staticmethod
    def _strcmp(a, b):
        if a == b:
            return 0
        elif a < b:
            return -1
        else:
            return 1


def start_broadcast(tuner_id):
    current_app.logger.debug("Starting broadcast for " + tuner_id)
    return runtime_utils.start_broadcast(tuner_id)


def stop_broadcast(tuner_id):
    current_app.logger.debug("Stopping broadcast for " + tuner_id)
    return runtime_utils.stop_broadcast(tuner_id)


def archive_tuner_configs(tuner_id):
    archive_path = os.path.join('/tmp', tuner_id + '.zip')
    with ZipFile(archive_path, 'w') as f:
        config_file = config_utils.get_config_file_path(tuner_id)
        if os.path.isfile(config_file):
            f.write(config_file, config_utils.TUNER_CONFIG_FILE_NAME)

        logging_config_file = config_utils.get_tuner_logging_config_path(
            tuner_id)
        if os.path.isfile(logging_config_file):
            f.write(logging_config_file,
                    config_utils.TUNER_LOGGING_CONFIG_FILE_NAME)

        feed_config_file = config_utils.get_feed_config_file_path(tuner_id)
        if os.path.isfile(feed_config_file):
            f.write(feed_config_file, config_utils.TUNER_FEED_CONFIG_FILE_NAME)

        schedules_path = config_utils.get_schedules_dir(tuner_id)
        if os.path.exists(schedules_path):
            schedules_name = os.path.basename(schedules_path)
            f.write(schedules_path, schedules_name)
            for (dirpath, dirnames, filenames) in os.walk(schedules_path):
                for filename in filenames:
                    if filename.endswith('.sch'):
                        f.write(os.path.join(dirpath, filename),
                                os.path.join(schedules_name, filename))
                break
    return archive_path

def archive_tuner_audio_files(tuner_id, audio_files):
    audio_dir = get_tuner_audio_dir(tuner_id)
    zip_file_stream =  zipstream.ZipFile(mode='w', compression=zipfile.ZIP_STORED)
    for file in audio_files:
        file_path = os.path.join(audio_dir, file['path'])
        if os.path.isfile(file_path):
            zip_file_stream.write(file_path, file['name'])
        elif os.path.isdir(file_path):
            filepath_len = file_path.rfind('/')
            for (dirpath, dirnames, filenames) in os.walk(file_path):
                dirname = dirpath[filepath_len:]
                for filename in filenames:
                    fs_path = os.path.join(dirpath, filename)
                    file_path_archive = os.path.join(dirname, filename)
                    zip_file_stream.write(fs_path, file_path_archive)

    return zip_file_stream

def add_new_tuner_from_archive(archive_path):
    logger = current_app.logger
    logger.debug("Creating new tuner from archive", archive_path)
    if not os.path.isfile(archive_path):
        raise ServiceMethodError("Archive file doesn't exist " + archive_path)

    base_dir = config_utils.get_new_tuner_tmp_dir()
    if os.path.exists(base_dir):
        shutil.rmtree(base_dir)
    else:
        os.makedirs(base_dir)

    with ZipFile(archive_path, 'r') as f:
        names = f.namelist()
        logger.debug("Archive members: %s", names)
        if config_utils.TUNER_CONFIG_FILE_NAME not in names:
            return None

    with ZipFile(archive_path, 'r') as f:
        names = f.namelist()
        logger.debug("Archive members: %s", names)
        if config_utils.TUNER_CONFIG_FILE_NAME in names:
            logger.debug("Extracting %s to %s",
                         config_utils.TUNER_CONFIG_FILE_NAME, base_dir)
            f.extract(config_utils.TUNER_CONFIG_FILE_NAME, base_dir)

        if config_utils.TUNER_FEED_CONFIG_FILE_NAME in names:
            logger.debug("Extracting %s to %s",
                         config_utils.TUNER_FEED_CONFIG_FILE_NAME, base_dir)
            f.extract(config_utils.TUNER_FEED_CONFIG_FILE_NAME, base_dir)

        schedules_dir_name = config_utils.get_schedules_dir_name()
        logger.debug("Creating schedules dir")
        os.mkdir(os.path.join(base_dir, 'schedules'))
        for name in names:
            if os.path.dirname(name) == schedules_dir_name and os.path.basename(name).endswith('.sch'):
                logger.debug("Extracting %s to %s", name, base_dir)
                f.extract(name, base_dir)
    tuner = Tuner.load_from_file('', False, True)
    return tuner


def apply_archived_tuner_configs(tuner_id, archive_path):
    logger = current_app.logger
    logger.debug("Uploading configuration for tuner %s", tuner_id)
    if not os.path.isfile(archive_path):
        raise ServiceMethodError("Archive file doesn't exist " + archive_path)
    base_dir = config_utils.get_tuner_dir(tuner_id)

    with ZipFile(archive_path, 'r') as f:
        names = f.namelist()
        logger.debug("Archive members: %s", names)
        if config_utils.TUNER_CONFIG_FILE_NAME not in names:
            return False

    logger.debug("Deleting old tuner configuration %s" % base_dir)
    shutil.rmtree(base_dir)
    os.mkdir(base_dir)

    with ZipFile(archive_path, 'r') as f:
        names = f.namelist()
        logger.debug("Archive members: %s", names)
        if config_utils.TUNER_CONFIG_FILE_NAME in names:
            logger.debug("Extracting %s to %s",
                         config_utils.TUNER_CONFIG_FILE_NAME, base_dir)
            f.extract(config_utils.TUNER_CONFIG_FILE_NAME, base_dir)

        if config_utils.TUNER_FEED_CONFIG_FILE_NAME in names:
            logger.debug("Extracting %s to %s",
                         config_utils.TUNER_FEED_CONFIG_FILE_NAME, base_dir)
            f.extract(config_utils.TUNER_FEED_CONFIG_FILE_NAME, base_dir)

        schedules_dir_name = config_utils.get_schedules_dir_name()
        logger.debug("Creating schedules dir")
        os.mkdir(config_utils.get_schedules_dir(tuner_id))
        for name in names:
            if os.path.dirname(name) == schedules_dir_name and os.path.basename(name).endswith('.sch'):
                logger.debug("Extracting %s to %s", name, base_dir)
                f.extract(name, base_dir)
    refresh_tuner(tuner_id)
    return True


def refresh_tuner(tuner_id):
    current_app.logger.debug('Refresh configuration for tuner %s', tuner_id)
    tuner = get_tuner(tuner_id)
#   create logging config
    tuner_service_utils.gen_logging_config(tuner_id)
    tuner.save_profile_file()
    tuner_service_utils.gen_init_script(
        tuner, config_utils.get_config_file_path(tuner_id))
    tuner_service_utils.restart_tuner(tuner_id)


def get_clenup_service():
    return CleanupService.load_from_file()


def update_cleanup_service(service):
    cmd = 'python /opt/tapperui/cleanup.py --config /etc/fmtapper/cleanup_service.conf'

    system_cron = CronTab(tabfile='/etc/crontab', user=False)
    cron_jobs = system_cron.find_command(cmd)
    for job in cron_jobs:
        system_cron.remove(job)

    cron_time = service.audio_check_interval
    m = re.search('\A(\d\d):(\d\d):(\d\d)\Z', cron_time)
    days = int(m.group(1))
    hours = int(m.group(2))
    minutes = int(m.group(3))

    cron_job = system_cron.new(command=cmd, user='root')
    update_cron_job(cron_job, minutes, hours, days)

    system_cron.write()
    service.save_to_file()


def get_cleanup_service_log_files():
    """
    Возвращает список объектов FileInfo для всех файлов логов сервиса очистки
    :return: список FileInfo
    """
    logger = current_app.logger
    logger.debug("Get log files for cleanup service")
    logs_dir = config_utils.get_cleanup_service_logs_dir()
    if not os.path.isdir(logs_dir):
        logger.warn("Cleanup service log directory does't exist")
        return []
    else:
        pattern = config_utils.get_cleanup_service_log_pattern()
        res = _get_files(logs_dir, False)
        res.children = filter(
            lambda c: c.name.startswith(pattern), res.children)
        return res


def get_tuners_status():
    info = []
    for tuner_id in get_tuner_ids():
        info.append(get_tuner_status(tuner_id))
    return info


def get_tuner_status(tuner_id):
    logger = current_app.logger
    logger.debug("Get tuner's runtime status: %s", tuner_id)
    try:
        res = RuntimeInfo(tuner_id=tuner_id)
        (res.status, res.err_num, res.err_str,
         res.ftp_status) = tuner_service_utils.get_tuner_status(tuner_id)
        t = Tuner.load_from_file(tuner_id)
        res.is_schedules_applied = (get_schedules(
            tuner_id).applied_count > 0 and t.schedule)
    except Exception, e:
        logger.exception("Can't get tuner's %s status: %s", tuner_id, e)
        return

    if res.status == Tuner.TunerStatus.RUNNING:
        try:
            res = runtime_utils.get_runtime_status(tuner_id, res)
        except Exception, e:
            logger.exception(
                "Can't get tuner's %s runtime info: %s", tuner_id, e)
    return res


def get_fmtapper_ver():
    current_app.logger.debug('Getting fmtapper version')
    version = '---'
    try:
        version = subprocess.check_output(['/usr/bin/fmtapper', '--version'])
        version = version.strip()
    except:
        pass
    return version


def get_tapperui_ver():
    current_app.logger.debug('Getting tapperui version')
    ps = subprocess.Popen('dpkg -s tapperui | grep Version | cut -c 10-', shell=True, stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT)
    output = ps.communicate()[0]
    if len(output) == 0 or 'not installed' in output:
        return '---'
    return output


def get_fmtapper_features():
    current_app.logger.debug('Getting fmtapper features')
    features = subprocess.check_output(
        ['/usr/bin/fmtapper', '--list-features'])
    features = features.strip('\n').split(',')
    return features


def send_file_partial(path):
    """
        Simple wrapper around send_file which handles HTTP 206 Partial Content
        (byte ranges)
        TODO: handle all send_file args, mirror send_file's error handling
        (if it has any)
    """
    range_header = request.headers.get('Range', None)
    if not range_header:
        return send_file(path, as_attachment=True)

    size = os.path.getsize(path)
    byte1, byte2 = 0, None

    m = re.search('(\d+)-(\d*)', range_header)
    g = m.groups()

    if g[0]: byte1 = int(g[0])
    if g[1]: byte2 = int(g[1])

    length = size - byte1
    if byte2 is not None:
        length = byte2 - byte1 + 1

    data = None
    with open(path, 'rb') as f:
        f.seek(byte1)
        data = f.read(length)

    rv = Response(data,
                  206,
                  mimetype=mimetypes.guess_type(path)[0],
                  direct_passthrough=True)
    rv.headers.add('Content-Range', 'bytes {0}-{1}/{2}'.format(byte1, byte1 + length - 1, size))
    rv.headers.add('Accept-Ranges', 'bytes')
    return rv


def is_ntp_active():
    try:
        # проверяем список доступных нтп-серверов
        output = subprocess.check_output(['ntpq', '-p']).decode().split('\n')
        for line in output:
            # текущий сервер синхронизации отмечен * в начале строки
            if line.startswith('*'):
                return True
    except:
        pass
    return False


from enum import Enum
class InitSystem(Enum):
    UPSTART = 1
    SYSTEMD = 2

def get_init_system_config():
    output = str(subprocess.check_output(['ls', '-l', '/sbin/init']))
    if 'systemd' in output:
        return SystemdConfig()
    return UpstartConfig()

def check_pattern_in_command_output(command, pattern):
    p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    retval = p.wait()
    if retval > 0:
        return
    status_str = ' '.join(map(str, p.stdout.readlines()))
    return pattern in status_str

class UpstartConfig:
    def __init__(self):
        self.init_type = InitSystem.UPSTART

    @property
    def init_folder(self):
        dest_dir = '/etc/init'
        return dest_dir

    @property
    def service_template_name(self):
        template_name = 'init_upstart_script.tpl'
        return template_name

    def get_status(self, service_name):
        command = ['status', service_name]
        pattern = 'start/running'
        return check_pattern_in_command_output(command, pattern)

    def enable_service(self, script_name):
        pass

    def disable_service(self, script_name):
        pass

    def script_name(self, tuner_id):
        template = "fmtapper_%s.conf"
        return template % tuner_id

    def start_service_command(self, service_name, start=True):
        if start:
            command = "start"
        else:
            command = "stop"
        command = '%s %s' % (command, service_name)
        return command

    def start_service(self,  service_name, start=True):
        command = self.start_service_command(service_name, start)
        return subprocess.call(command, shell=True)


class SystemdConfig(UpstartConfig):
    def __init__(self):
        self.init_type = InitSystem.SYSTEMD

    @property
    def init_folder(self):
        dest_dir = '/etc/systemd/system'
        return dest_dir

    @property
    def service_template_name(self):
        template_name = 'init_systemd_script.tpl'
        return template_name

    def get_status(self, service_name):
        command = ['systemctl', 'status', service_name]
        pattern = 'active (running)'
        return check_pattern_in_command_output(command, pattern)

    def enable_service(self, script_name):
        subprocess.call("systemctl daemon-reload", shell=True)
        subprocess.call("systemctl enable " + script_name, shell=True)

    def disable_service(self, script_name):
        subprocess.call("systemctl disable " + script_name, shell=True)

    def script_name(self, tuner_id):
        template = "fmtapper_%s.service"
        return template % tuner_id

    def start_service_command(self, service_name, start=True):
        command = "systemctl %s %s" % ('start' if start else 'stop', service_name)
        return command