# -*- coding: utf-8 -*-
import sqlite3
import re
from tuner import DeviceConfig, Tuner
from string import strip
from flask import current_app
import os


def connect_to_db(database):
    def make_dicts(cur, row):
        return dict((cur.description[idx][0], value)
                    for idx, value in enumerate(row))

    db = sqlite3.connect(database)
    db.row_factory = make_dicts
    return db


def query_db(db, query, args=(), one=False):
    """
    Query database with given args
    :param db: database object
    :param query: query string
    :param args: query args
    :param one: unique result
    :return: result
    """
    cur = db.execute(query, args)
    rv = cur.fetchall()
    cur.close()
    return (rv[0] if rv else None) if one else rv


def parse_audio_devices_output(lines):
    logger = current_app.logger
    res = []
    name_regex = re.compile('\A\d+\.\s+"(.*)"')
    input_regex = re.compile('Max input channels:\s+(\d+)')
    sample_rate_regex = re.compile('Default sample rate:\s+(\d+)')
    name = None
    input_channels = 0
    sample_rate = 0
    for line in lines:
        if name is None:
            m = name_regex.search(line)
            if m:
                name = m.group(1)
                logger.debug('Found audio device %s', name)
        elif len(strip(line)) == 0:
            device = DeviceConfig(name=name, sample_rate=sample_rate)
            device.input_channels = input_channels
            res.append(device)
            name = None
            sample_rate = 0
            input_channels = 0
        else:
            m = input_regex.search(line)
            if m:
                input_channels = int(m.group(1))
            else:
                m = sample_rate_regex.search(line)
                if m:
                    sample_rate = int(m.group(1))
    available = filter(lambda i: i.input_channels > 0, res)
    parsed = []
    for device in available:
        try:
            if _is_managed_audio_device(device.name):
                device.dev_type = Tuner.DeviceType.USB_TUNER
            else:
                device.dev_type = Tuner.DeviceType.AUDIO_DEVICE
            parsed.append(device)
        except Exception, e:
            logger.error('Can not determine device type', e)
    return parsed


def _is_managed_audio_device(name):
    return get_radio_device_name(name) is not None


def get_radio_device_name(name):
    logger = current_app.logger
    (card_num, card_dev_num) = _parse_device_name(name)
    res = _get_usb_path(card_num, card_dev_num)
    if res is not None:
        (bus_num, dev_num) = res
        logger.debug('Found bus %s, device %s', bus_num, dev_num)
        bus_path = "/sys/bus/usb/devices/%s-%s" % (bus_num, dev_num)
        for (dirpath, dirnames, filenames) in os.walk(bus_path):
            logger.debug("Probing %s", dirpath)
            name = os.path.basename(dirpath)
            radio_name = os.path.join('/dev', name)
            if name.startswith("radio") and os.path.exists(radio_name):
                logger.debug("Found radio device %s", name)
                return radio_name
            dirnames[:] = [dirname for dirname in dirnames if _is_searh_radio_dirname(dirname)]
        logger.debug("No radio device found")
        return None
    else:
        return None


def _is_searh_radio_dirname(dirname):
    return dirname == 'video4linux' or dirname.startswith('card') or dirname.startswith('radio') or \
        re.match('\A[\d:\.-]+\Z', dirname)


def _get_usb_path(card_num, dev_num):
    """
    Возвращает пару (номер шины, номер устройства) для ALSA устройства, если такое найдено в списке USB устройств
    :param card_num: номер карты
    :param dev_num: номер устройства
    :return: None или (номер шины, номер устройства)
    """
    logger = current_app.logger
    card_dev_name = "pcmC%sD%sc" % (card_num, dev_num)
    logger.debug("Searching card device %s", card_dev_name)
    for (dirpath, dirnames, filenames) in os.walk('/sys/bus/usb/devices', topdown=True, followlinks=True):
        logger.debug("Probing %s", dirpath)
        if os.path.basename(dirpath) == card_dev_name:
            return _extract_usb_path(dirpath)
        dirnames[:] = [dirname for dirname in dirnames if _is_searh_dirname(dirname, card_dev_name)]
    return None


def _is_searh_dirname(dirname, dev_name):
    return dirname == 'sound' or dirname.startswith('card') or dirname == dev_name or \
        re.match('\A[\d:\.-]+\Z', dirname) is not None


def _extract_usb_path(path):
    tmp = path.replace('/sys/bus/usb/devices/', '')
    ind = tmp.index('/')
    if ind > 0:
        tmp = tmp[:ind]
        m = re.match('\A(\d+)-(\d+)\Z', tmp)
        if m is not None:
            return m.group(1), m.group(2)
        else:
            return None
    else:
        return None


def _parse_device_name(name):
    m = re.match('\A.*\(hw:(\d+)\s*,(\d+)\)\Z', name)
    if m is not None:
        return m.group(1), m.group(2)
    else:
        raise Exception("Invalid device name: " + name)
